package Grenouille::Log;

# This software may be used and distributed according to the terms
# of the GNU General Public License, incorporated herein by reference.

use strict;
use Exporter qw();
use Time::HiRes qw(time);
use Grenouille::Vars qw($LOGFILE $MIN @MONTHS);
use Grenouille::Scheduler qw(schedule_event);
use vars qw(@ISA @EXPORT_OK 
            %LOG_PRIORITY $LOG_LEVEL $DISPLAY_LEVEL);
@ISA=qw(Exporter);
@EXPORT_OK=qw(open_log mark_log close_log display_and_log
              set_log_level set_verbose_level);

%LOG_PRIORITY = ('DEBUG'=>1, 'INFO'=>2,
                 'NOTICE'=>3, 'DISPLAY'=>4, 'LOG'=>5,
                 'WARNING'=>6, 'ERROR'=>7);

$LOG_LEVEL = $LOG_PRIORITY{'LOG'};
$DISPLAY_LEVEL = $LOG_PRIORITY{'INFO'};

sub open_log() {
    if($LOGFILE) {
	unless (-w $LOGFILE) {
	    Grenouille::System::swap_privileges();
	    eval {
		open(LOG, ">>$LOGFILE") or die "$LOGFILE: $!";
		close LOG;
		chown $<, -1, $LOGFILE;
		chmod 0755, $LOGFILE;
	    };
	    Grenouille::System::swap_privileges();
	}
	# logging is important for security, give up if logging is not
	# available - undef $LOGFILE if you really mean it
	open(LOG, ">>$LOGFILE") or die "$LOGFILE: $!";
	autoflush LOG 1;
	_log('-- log start --');
	schedule_event(time() + 20 * $MIN, \&mark_log);
    }
}

sub mark_log($) { # EVENT
    my($schedule_time) = shift;
    _log('-- mark --');
    schedule_event($schedule_time + 20 * $MIN, \&mark_log);
}

sub close_log() {
    {
	# avoid a stat() on unopened filehandle if the log was never used
	local $SIG{__WARN__} = sub { };
	if($LOGFILE and stat LOG) {
	    _log('-- log end --');
	    close LOG;
	}
    }
}

sub _log($) {
    my($msg) = @_;
    my($sec, $min, $hour, $mday, $mon, 
       undef, undef, undef, undef) = localtime();
    printf(LOG "%s %2s %02d:%02d:%02d %s\n", $MONTHS[$mon], $mday, 
                                             $hour, $min, $sec, $msg);
}

my (%DISPLAY_PREFIX) = ('DISPLAY'=>'', 'NOTICE'=>'');
my (%LOG_PREFIX) = ('LOG'=>'', 'NOTICE'=>'');

sub display_and_log($$) {
    my($level, $msg) = @_;
    my($nlevel) = $LOG_PRIORITY{$level};
    die "unknown priority $level" unless $nlevel;
    if(($nlevel != $LOG_PRIORITY{'LOG'}) and 
       (($nlevel == $LOG_PRIORITY{'DISPLAY'}) or 
        ($nlevel >= $DISPLAY_LEVEL))) {
	my($prefixed_msg) = (exists $DISPLAY_PREFIX{$level} ? $DISPLAY_PREFIX{$level} : "[$level] ") . $msg;
	if($nlevel > $LOG_PRIORITY{'NOTICE'}) {
	    print STDERR "$prefixed_msg\n";
	} else {
	    print "$prefixed_msg\n";
	}
    }
    if($LOGFILE and ($nlevel != $LOG_PRIORITY{'DISPLAY'}) and 
       (($nlevel == $LOG_PRIORITY{'LOG'}) or ($nlevel >= $LOG_LEVEL))) {
	_log((exists $LOG_PREFIX{$level} ? $LOG_PREFIX{$level} : "[$level] ") . $msg);
    }
}

sub set_log_level($$) {
  my($opt, $level) = @_;
  $LOG_LEVEL = $LOG_PRIORITY{uc($level)};
  die "Priorit $level inconnue" unless $LOG_LEVEL;
}

sub set_verbose_level($$) {
  my($opt, $level) = @_;
  $DISPLAY_LEVEL = $LOG_PRIORITY{uc($level)};
  die "Priorit $level inconnue" unless $DISPLAY_LEVEL;
}

1;
